posx = 0
posz = -40
rolloff = 0


local currModel = 3
local models = {
  "none",
  "inverse",
  "inverse clamped",
  "linear",
  "linear clamped",
  "exponent",
  "exponent clamped"
}

function love.load()
  source = love.audio.newSource("pickup_mono.ogg", "static")
  source:setLooping(true)
  source:play()
  source:setRolloff(0.01)
  rolloff = source:getRolloff()
  print(love.audio.getDistanceModel()) -- Default is inverse clamped

  love.audio.setDistanceModel(models[currModel])

  -- Listner is at 0 0 0
  love.audio.setPosition(0, 0, posz)
end

function love.update(dt)
  local offset = 1
  if love.keyboard.isDown("lshift") then
    offset = 10
  end

  if love.keyboard.isDown("left") then
    posx = posx - offset
  end
  if love.keyboard.isDown("right") then
    posx = posx + offset
  end

  if love.keyboard.isDown("up") then
    posz = posz - offset
  end
  if love.keyboard.isDown("down") then
    posz = posz + offset
  end
  --love.audio.setPosition(posx/100, posy/100, 0)
  -- edge of the screen should be 100
  local scaledPx = posx / (love.graphics.getWidth()/2) * 100
  source:setPosition(scaledPx, 0, 0)

  love.audio.setPosition(0, 0, posz)
end

function love.keypressed(key, code)
  if key == "[" then
    rolloff = rolloff + 0.01
    source:setRolloff(rolloff)
  end
  if key == "]" then
    rolloff = rolloff - 0.01
    if rolloff < 0 then rolloff = 0 end
    source:setRolloff(rolloff)
  end
  if key == "=" or key == "-" then
    if key == "=" then
      currModel = currModel+1
      if currModel > #models then currModel = 1 end
    end
    if key == "-" then
      currModel = currModel-1
      if currModel == 0 then currModel = #models end
    end
    love.audio.setDistanceModel(models[currModel])
    source:stop()
    source:play()
  end
  
  -- Haha fake doppler
  if key == "." then source:setVelocity(10, 0, 0) end
  if key == "," then source:setVelocity(-10, 0, 0) end
end

function love.draw()
  love.graphics.setPointSize(10)
  love.graphics.point(posx + love.graphics.getWidth()/2, love.graphics.getHeight()/2)
  local x, y, z = source:getPosition()
  love.graphics.print("Sound position: " .. x .. ", " .. y .. ", " .. z, 10, 10)
  love.graphics.print("Sound rolloff: " .. source:getRolloff(), 10, 30)
  love.graphics.print("Distance model: " .. love.audio.getDistanceModel(), 10, 50)

  love.graphics.printf("left/right moves sound source\n" ..
                       "up/down moves listener in/out of screen\n" ..
                       "hold shift to move faster\n" ..
                       "+/- to change between distance models\n" ..
                       "[/] to increase/decrease rolloff", love.graphics.getWidth()-280, 10, 270)

  love.graphics.print("Listener at: 0, 0, " .. posz, 10, 80)
end
